package com.flyfish.framework.beans.meta;

import com.flyfish.framework.annotations.Property;
import lombok.Data;
import org.apache.commons.lang3.reflect.FieldUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.core.annotation.AnnotationUtils;

import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;
import java.util.*;
import java.util.stream.Collectors;

/**
 * bean的属性描述
 * 对应js的类型
 *
 * @author wangyu
 */
@Data
public class BeanProperty {

    // 属性名称
    private String name;

    // 属性标题
    private String title;

    // 描述信息
    private String description;

    // bean属性的类型，js类型
    private BeanPropertyType type;

    // 属性
    private Map<String, Object> props;

    // 类型为object时，拥有子表单
    private List<BeanProperty> children;

    /**
     * 类加载并解析属性
     *
     * @return 结果
     */
    public static List<BeanProperty> from(Class<?> clazz) {
        PropertyDescriptor[] descriptors = BeanUtils.getPropertyDescriptors(clazz);
        return Arrays.stream(descriptors)
                .filter(descriptor -> !"class".equals(descriptor.getName()))
                .map(descriptor -> BeanProperty.form(descriptor, clazz))
                .collect(Collectors.toList());
    }

    /**
     * 来自属性解释器构造
     * 支持对象嵌套
     *
     * @param descriptor 解释器
     * @return 结果
     */
    public static BeanProperty form(PropertyDescriptor descriptor, Class<?> beanClass) {
        // 尝试获取field
        Field field = FieldUtils.getField(beanClass, descriptor.getName(), true);
        // 存在field，可以干一些坏事
        BeanProperty property = new BeanProperty();
        property.setName(descriptor.getName());
        property.setType(BeanPropertyType.of(descriptor, beanClass));
        if (null != field) {
            Property props = AnnotationUtils.findAnnotation(field, Property.class);
            if (null != props) {
                String parentName = Optional.ofNullable(beanClass.getAnnotation(RestBean.class)).map(RestBean::name).orElse("");
                property.setTitle(props.inherited() ? parentName + props.title() : props.title());
                property.setDescription(props.description());
            }
        }
        Class<?> clazz = descriptor.getPropertyType();
        if (property.getType() == BeanPropertyType.OBJECT) {
            // 有子bean注解才处理
            if (null != field && field.isAnnotationPresent(SubBean.class)) {
                property.setChildren(from(clazz));
            }
        } else if (property.getType() == BeanPropertyType.DB_REF) {
            // 当存在db-ref时，解析为动态数据源
            RestBean restBean = clazz.getAnnotation(RestBean.class);
            if (null != restBean) {
                String uri = restBean.value();
                Map<String, Object> props = new HashMap<>();
                props.put("uri", uri);
                property.setProps(props);
            }
        }
        return property;
    }
}
